import builtins
import contextlib
import unittest
from unittest import mock

from uvcclient import store


class OpenHelper:
    def __init__(self):
        self.mock = mock.MagicMock()

    @contextlib.contextmanager
    def fake_open(self, path, mode):
        self.path = path
        self.mode = mode
        yield self.mock


class TestStore(unittest.TestCase):
    @mock.patch.object(builtins, "open")
    @mock.patch("os.path.expanduser")
    def test_loads_correct_file_default(self, mock_expand, mock_open):
        mock_expand.return_value = "foobar"
        mock_open.side_effect = OSError
        store.InfoStore()
        mock_open.assert_called_once_with("foobar")

    @mock.patch.object(builtins, "open")
    @mock.patch("os.path.expanduser")
    def test_loads_correct_file(self, mock_expand, mock_open):
        mock_open.side_effect = OSError
        store.InfoStore("barfoo")
        self.assertFalse(mock_expand.called)
        mock_open.assert_called_once_with("barfoo")

    @mock.patch("os.chmod")
    def test_writes_correct_file(self, mock_chmod):
        with mock.patch.object(builtins, "open") as mock_open:
            mock_open.side_effect = OSError
            s = store.InfoStore("barfoo")
        opener = OpenHelper()
        with mock.patch.object(builtins, "open", new=opener.fake_open):
            s.save()
        self.assertTrue(opener.mock.write.called)
        mock_chmod.assert_called_once_with("barfoo", 0o600)

    def test_get_camera_passwords(self):
        with mock.patch.object(builtins, "open") as mock_open:
            mock_open.side_effect = OSError
            s = store.InfoStore("barfoo")
        s._data = {"camera_passwords": {"foo": "bar"}}
        self.assertEqual({"foo": "bar"}, s.get_camera_passwords())
        self.assertEqual("bar", s.get_camera_password("foo"))
        self.assertEqual(None, s.get_camera_password("doesnotexist"))

    def test_set_camera_password(self):
        with mock.patch.object(builtins, "open") as mock_open:
            mock_open.side_effect = OSError
            s = store.InfoStore("barfoo")
        with mock.patch.object(s, "save") as mock_save:
            s.set_camera_password("foo", "bar")
            mock_save.assert_called_once_with()
        self.assertEqual({"foo": "bar"}, s.get_camera_passwords())



import json
import unittest
import zlib
from http import client as httplib
from unittest import mock

from uvcclient import nvr


class TestClientLowLevel(unittest.TestCase):
    def setUp(self):
        super().setUp()
        self._patches = []
        try:
            import httplib  # noqa: F401

            http_mock = mock.patch("httplib.HTTPConnection")
        except ImportError:
            http_mock = mock.patch("http.client.HTTPConnection")
        http_mock.start()
        self._patches.append(http_mock)
        bootstrap_mock = mock.patch.object(
            nvr.UVCRemote, "_get_bootstrap", side_effect=self._bootstrap
        )
        bootstrap_mock.start()
        self._patches.append(bootstrap_mock)

    def _bootstrap(self):
        return {"systemInfo": {"version": "3.1.3"}}

    def cleanUp(self):
        for i in self._patches:
            i.stop()

    def test_uvc_request_get(self):
        client = nvr.UVCRemote("foo", 7080, "key")
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 200
        resp.read.return_value = json.dumps({}).encode()
        client._uvc_request("/bar")
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Accept-Encoding": "gzip, deflate, sdch",
        }
        conn.request.assert_called_once_with("GET", "/bar?apiKey=key", None, headers)

    def test_uvc_request_put(self):
        client = nvr.UVCRemote("foo", 7080, "key")
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 200
        resp.read.return_value = json.dumps({}).encode()
        result = client._uvc_request("/bar?foo=bar", method="PUT", data={"foo": "bar"})
        self.assertEqual({}, result)
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Accept-Encoding": "gzip, deflate, sdch",
        }
        conn.request.assert_called_once_with(
            "PUT", "/bar?foo=bar&apiKey=key", '{"foo": "bar"}', headers
        )

    def test_uvc_request_failed(self):
        client = nvr.UVCRemote("foo", 7080, "key")
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 404
        self.assertRaises(
            nvr.NvrError, client._uvc_request, "/bar", method="PUT", data={"foo": "bar"}
        )

    def test_uvc_request_failed_noauth(self):
        client = nvr.UVCRemote("foo", 7080, "key")
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 401
        self.assertRaises(
            nvr.NotAuthorized,
            client._uvc_request,
            "/bar",
            method="PUT",
            data={"foo": "bar"},
        )

    def test_uvc_request_deflated(self):
        client = nvr.UVCRemote("foo", 7080, "key")
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 200
        resp.read.return_value = zlib.compress(json.dumps({}).encode())
        resp.getheaders.return_value = [("Content-Encoding", "gzip")]
        client._uvc_request("/bar")
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Accept-Encoding": "gzip, deflate, sdch",
        }
        conn.request.assert_called_once_with("GET", "/bar?apiKey=key", None, headers)


class TestClient32(unittest.TestCase):
    @mock.patch.object(nvr.UVCRemote, "_get_bootstrap")
    def test_bootstrap_server_version(self, mock_bootstrap):
        mock_bootstrap.return_value = {"systemInfo": {"version": "3.4.5"}}
        client = nvr.UVCRemote("foo", 7080, "key")
        self.assertEqual((3, 4, 5), client.server_version)

    @mock.patch.object(nvr.UVCRemote, "_get_bootstrap")
    def test_bootstrap_server_version_beta(self, mock_bootstrap):
        mock_bootstrap.return_value = {"systemInfo": {"version": "3.4.beta5"}}
        client = nvr.UVCRemote("foo", 7080, "key")
        self.assertEqual((3, 4, 0), client.server_version)

    @mock.patch.object(nvr.UVCRemote, "_get_bootstrap")
    @mock.patch.object(nvr.UVCRemote, "index")
    def test_310_returns_uuid(self, mock_index, mock_bootstrap):
        mock_index.return_value = [
            {
                "name": mock.sentinel.name,
                "uuid": mock.sentinel.uuid,
                "id": mock.sentinel.id,
            }
        ]
        mock_bootstrap.return_value = {"systemInfo": {"version": "3.1.0"}}
        client = nvr.UVCRemote("foo", 7080, "key")
        self.assertEqual(mock.sentinel.uuid, client.name_to_uuid(mock.sentinel.name))

    @mock.patch.object(nvr.UVCRemote, "_get_bootstrap")
    @mock.patch.object(nvr.UVCRemote, "index")
    def test_320_returns_uuid(self, mock_index, mock_bootstrap):
        mock_index.return_value = [
            {
                "name": mock.sentinel.name,
                "uuid": mock.sentinel.uuid,
                "id": mock.sentinel.id,
            }
        ]
        mock_bootstrap.return_value = {"systemInfo": {"version": "3.2.0"}}
        client = nvr.UVCRemote("foo", 7080, "key")
        self.assertEqual(mock.sentinel.id, client.name_to_uuid(mock.sentinel.name))


class TestClient(unittest.TestCase):
    def setUp(self):
        super().setUp()
        self._patches = []
        try:
            import httplib  # noqa: F401

            http_mock = mock.patch("httplib.HTTPConnection")
        except ImportError:
            http_mock = mock.patch("http.client.HTTPConnection")
        http_mock.start()
        self._patches.append(http_mock)
        bootstrap_mock = mock.patch.object(
            nvr.UVCRemote, "_get_bootstrap", side_effect=self._bootstrap
        )
        bootstrap_mock.start()
        self._patches.append(bootstrap_mock)

    def _bootstrap(self):
        return {"systemInfo": {"version": "3.1.3"}}

    def cleanUp(self):
        for i in self._patches:
            i.stop()

    def test_set_recordmode(self):
        fake_resp1 = {
            "data": [
                {
                    "recordingSettings": {
                        "fullTimeRecordEnabled": False,
                        "motionRecordEnabled": False,
                    }
                }
            ]
        }
        fake_resp2 = {
            "data": [
                {
                    "recordingSettings": {
                        "fullTimeRecordEnabled": True,
                        "motionRecordEnabled": False,
                        "channel": 1,
                    }
                }
            ]
        }

        def fake_req(path, method="GET", data=None):
            if method == "GET":
                return fake_resp1
            elif method == "PUT":
                self.assertEqual(json.dumps(fake_resp2["data"][0]), data)
                return fake_resp2

        client = nvr.UVCRemote("foo", 7080, "key")
        with mock.patch.object(client, "_uvc_request") as mock_r:
            mock_r.side_effect = fake_req
            client.set_recordmode("uuid", "full", chan="medium")
            self.assertTrue(mock_r.called)

        fake_resp2["data"][0]["recordingSettings"] = {
            "fullTimeRecordEnabled": False,
            "motionRecordEnabled": True,
            "channel": 0,
        }
        with mock.patch.object(client, "_uvc_request") as mock_r:
            mock_r.side_effect = fake_req
            client.set_recordmode("uuid", "motion", chan="high")
            self.assertTrue(mock_r.called)

    def test_get_picture_settings(self):
        fake_resp = {"data": [{"ispSettings": {"settingA": 1, "settingB": "foo"}}]}
        client = nvr.UVCRemote("foo", 7080, "key")
        with mock.patch.object(client, "_uvc_request") as mock_r:
            mock_r.return_value = fake_resp
            self.assertEqual(
                {"settingA": 1, "settingB": "foo"}, client.get_picture_settings("uuid")
            )

    def test_set_picture_settings(self):
        fake_resp = {"data": [{"ispSettings": {"settingA": 1, "settingB": "foo"}}]}
        client = nvr.UVCRemote("foo", 7080, "key")
        newvals = {"settingA": 2, "settingB": "foo"}
        with mock.patch.object(client, "_uvc_request") as mock_r:
            mock_r.return_value = fake_resp
            resp = client.set_picture_settings("uuid", newvals)
            mock_r.assert_any_call(
                "/api/2.0/camera/uuid", "PUT", json.dumps({"ispSettings": newvals})
            )
            self.assertEqual(fake_resp["data"][0]["ispSettings"], resp)

    def test_set_picture_settings_coerces(self):
        fake_resp = {
            "data": [
                {
                    "ispSettings": {
                        "settingA": 1,
                        "settingB": "foo",
                        "settingC": False,
                    }
                }
            ]
        }
        client = nvr.UVCRemote("foo", 7080, "key")
        newvals = {"settingA": "2", "settingB": False, "settingC": "foo"}
        newvals_expected = {"settingA": 2, "settingB": "False", "settingC": True}
        with mock.patch.object(client, "_uvc_request") as mock_r:
            mock_r.return_value = fake_resp
            resp = client.set_picture_settings("uuid", newvals)
            mock_r.assert_any_call(
                "/api/2.0/camera/uuid",
                "PUT",
                json.dumps({"ispSettings": newvals_expected}),
            )
            self.assertEqual(fake_resp["data"][0]["ispSettings"], resp)

    def test_get_zones(self):
        fake_resp = {"data": [{"zones": ["fake-zone1", "fake-zone2"]}]}
        client = nvr.UVCRemote("foo", 7080, "key")
        with mock.patch.object(client, "_uvc_request") as mock_r:
            mock_r.return_value = fake_resp
            resp = client.list_zones("uuid")
            mock_r.assert_any_call("/api/2.0/camera/uuid")
            self.assertEqual(fake_resp["data"][0]["zones"], resp)

    def test_prune_zones(self):
        fake_resp = {"data": [{"zones": ["fake-zone1", "fake-zone2"]}]}
        client = nvr.UVCRemote("foo", 7080, "key")
        with mock.patch.object(client, "_uvc_request") as mock_r:
            mock_r.return_value = fake_resp
            client.prune_zones("uuid")
            mock_r.assert_any_call(
                "/api/2.0/camera/uuid", "PUT", json.dumps({"zones": ["fake-zone1"]})
            )

    def test_get_snapshot(self):
        client = nvr.UVCRemote("foo", 7080, "key")
        with mock.patch.object(client, "_safe_request") as mock_r:
            mock_r.return_value.status = 200
            mock_r.return_value.read.return_value = "image"
            resp = client.get_snapshot("foo")
            mock_r.assert_called_once_with(
                "GET", "/api/2.0/snapshot/camera/foo?force=true&apiKey=key"
            )
            self.assertEqual("image", resp)

    def test_get_snapshot_error(self):
        client = nvr.UVCRemote("foo", 7080, "key")
        with mock.patch.object(client, "_safe_request") as mock_r:
            mock_r.return_value.status = 401
            self.assertRaises(nvr.NvrError, client.get_snapshot, "foo")



import unittest
from unittest import mock

from uvcclient import nvr


class TestCliUtils(unittest.TestCase):
    FAKE_ENV1 = {
        "UVC": "http://192.168.1.1:7080/?apiKey=foo",
    }

    FAKE_ENV2 = {
        "UVC_HOST": "192.168.1.2",
        "UVC_PORT": "7443",
        "UVC_APIKEY": "myKey",
    }

    @mock.patch("os.getenv")
    def test_get_auth_combined(self, mock_getenv):
        mock_getenv.side_effect = self.FAKE_ENV1.get
        host, port, key, path = nvr.get_auth_from_env()
        self.assertEqual("192.168.1.1", host)
        self.assertEqual(7080, port)
        self.assertEqual("foo", key)
        self.assertEqual("/", path)

    @mock.patch("os.getenv")
    def test_get_separate(self, mock_getenv):
        mock_getenv.side_effect = self.FAKE_ENV2.get
        host, port, key, path = nvr.get_auth_from_env()
        self.assertEqual("192.168.1.2", host)
        self.assertEqual(7443, port)
        self.assertEqual("myKey", key)
        self.assertEqual("/", path)



import json
import unittest
from http import client as httplib
from unittest import mock

from uvcclient import camera


class TestCamera(unittest.TestCase):
    def test_set_led_on(self):
        c = camera.UVCCameraClient("foo", "ubnt", "ubnt")
        with mock.patch.object(c, "_cfgwrite") as mock_write:
            c.set_led(False)
            mock_write.assert_called_once_with("led.front.status", 0)

    def test_set_led_off(self):
        c = camera.UVCCameraClient("foo", "ubnt", "ubnt")
        with mock.patch.object(c, "_cfgwrite") as mock_write:
            c.set_led(True)
            mock_write.assert_called_once_with("led.front.status", 1)

    def test_get_snapshot(self):
        c = camera.UVCCameraClient("foo", "ubnt", "ubnt")
        with mock.patch.object(httplib, "HTTPConnection") as mock_conn:
            conn = mock_conn.return_value
            conn.getresponse.return_value.status = 200
            r = c.get_snapshot()
            self.assertEqual(conn.getresponse.return_value.read.return_value, r)

    def test_cfgwrite(self):
        c = camera.UVCCameraClient("foo", "ubnt", "ubnt")
        c._cookie = "foo-cookie"
        with mock.patch.object(httplib, "HTTPConnection") as mock_h:
            conn = mock_h.return_value
            conn.getresponse.return_value.status = 200
            self.assertTrue(c._cfgwrite("foo", "bar"))
            headers = {"Cookie": "foo-cookie"}
            conn.request.assert_called_once_with(
                "GET", "/cfgwrite.cgi?foo=bar", headers=headers
            )

    @mock.patch.object(httplib, "HTTPConnection")
    def test_login_v310(self, mock_h):
        c = camera.UVCCameraClient("foo", "ubnt", "ubnt")
        first = mock.MagicMock()
        second = mock.MagicMock()
        counter = [0]

        def fake_conn(*a, **k):
            if counter[0] == 0:
                counter[0] += 1
                return first
            elif counter[0] == 1:
                counter[0] += 1
                return second

        mock_h.side_effect = fake_conn
        cookie = "thecookie AIROS_SESSIONID=foo; bar"
        first.getresponse.return_value.getheaders.return_value = [
            ("Set-Cookie", cookie)
        ]
        second.getresponse.return_value.status = 200
        c.login()
        first.request.assert_called_once_with("GET", "/")
        self.assertEqual("POST", second.request.call_args_list[0][0][0])
        self.assertEqual("/login.cgi", second.request.call_args_list[0][0][1])
        formdata = "AIROS_SESSIONID=foo&password=ubnt&username=ubnt"
        self.assertEqual(
            sorted(formdata.split("&")),
            sorted(second.request.call_args_list[0][0][2].split("&")),
        )
        self.assertEqual(cookie, second.request.call_args_list[0][0][3]["Cookie"])

    @mock.patch.object(httplib, "HTTPConnection")
    def test_login_v320(self, mock_h):
        c = camera.UVCCameraClientV320("foo", "ubnt", "ubnt")
        resp = mock_h.return_value.getresponse.return_value
        resp.status = 200
        resp.getheaders.return_value = {"set-cookie": "cookie"}
        c.login()
        self.assertEqual("POST", mock_h.return_value.request.call_args_list[0][0][0])
        self.assertEqual(
            "/api/1.1/login", mock_h.return_value.request.call_args_list[0][0][1]
        )
        data = json.loads(mock_h.return_value.request.call_args_list[0][0][2])
        self.assertEqual({"username": "ubnt", "password": "ubnt"}, data)
        self.assertEqual("cookie", c._cookie)
